from rest_framework import viewsets

from ..utilities.pagination import MyPageNumberPagination
from ..utilities.permissions import {app_name}Permission
from rest_framework.filters import SearchFilter,OrderingFilter
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework_simplejwt.authentication import JWTAuthentication


from rest_framework.pagination import PageNumberPagination

class MyPageNumberPagination(PageNumberPagination):
    page_size = 20
    page_query_param = 'page'
    page_size_query_param = 'size'
    max_page_size = 5000


from django.core.management.base import BaseCommand, CommandError
# from polls.models import Question as Poll
from .utilities import app_management
from .utilities.auto_api import AutoApi


class Command(BaseCommand):
    help = "Automatically generate API components for specified apps"

    def add_arguments(self, parser):
        parser.add_argument("--app", nargs="+", type=str, help="Specify app names to generate APIs for")

    def handle(self, *args, **options):
        all_apps = app_management.get_custom_apps()
        if options.get('app'):
            for app in options["app"]:
                if app in all_apps:
                    auto_api_obj = AutoApi(app)
                    auto_api_obj.make_apis()
                    auto_api_obj.make_app_router()
                    self.stdout.write(self.style.SUCCESS(f'Successfully made API for app "{app}"'))
                else:
                    raise CommandError(f'{app} does not exist!')
        else:
            self.stdout.write(self.style.WARNING("No apps specified. Provide app names with --app"))



from rest_framework import viewsets
from rest_framework.filters import SearchFilter, OrderingFilter
from django_filters.rest_framework import DjangoFilterBackend
from ..models import {model_name}
from ..serializers.{model_serializer} import {model_list_serializers}, {model_retrieve_serializers}, {model_write_serializers}
from ..utilities.importbase import *

class {viewset_name}(viewsets.ModelViewSet):
    serializer_class = {model_list_serializers}
    # permission_classes = [{app_name}Permission]
    # authentication_classes = [JWTAuthentication]
    pagination_class = MyPageNumberPagination
    queryset = {model_name}.objects.all()

    filter_backends = [SearchFilter, DjangoFilterBackend, OrderingFilter]
    search_fields = ['id']
    ordering_fields = ['id']

    # filterset_fields = {{
    #     'id': ['exact'],
    # }}

    def get_queryset(self):
        queryset = super().get_queryset()
        return queryset.filter(user_id=self.request.user.id)

    def get_serializer_class(self):
        if self.action in ['create', 'update', 'partial_update']:
            return {model_write_serializers}
        elif self.action == 'retrieve':
            return {model_retrieve_serializers}
        return super().get_serializer_class()

    # @action(detail=False, methods=['get'], name="action_name", url_path="url_path")
    # def action_name(self, request, *args, **kwargs):
    #     return super().list(request, *args, **kwargs)




import os
from django.core.management.base import BaseCommand, CommandError
from .helper.helper import ViewSetsHelper

class BaseHelper(BaseCommand):
    def __init__(self, app_name, task_type, model_name, utilities=None):
        super().__init__()
        self.app_name = app_name
        self.type = task_type
        self.model = model_name
        self.utilities = utilities
        self.file_extension = '.py'
        self.backup_types = ['routers']

        self.initialize_paths()
        self.initialize_names()

    def initialize_paths(self):
        if self.utilities:
            self.model_type_stru = os.path.join(self.app_name, self.utilities, f"{self.type}{self.file_extension}")
            self.model_type_data_file = os.path.join('autoapi', 'management', 'commands', 'data', self.utilities, f"{self.type}{self.file_extension}")
            self.model_type_date_file_backup = os.path.join('autoapi', 'management', 'commands', 'data', self.utilities, f"backup{self.file_extension}")
        else:
            self.model_type_stru = os.path.join(self.app_name, self.type, f"{self.model.lower()}_{self.type}{self.file_extension}")
            if self.type in self.backup_types:
                self.model_type_stru = os.path.join(self.app_name, self.type, f"{self.type}{self.file_extension}")
            self.model_type_data_file = os.path.join('autoapi', 'management', 'commands', 'data', self.type, f"{self.type}{self.file_extension}")
            self.model_type_date_file_backup = os.path.join('autoapi', 'management', 'commands', 'data', self.type, f"backup{self.file_extension}")
            self.model_type_date_file_backup_import = os.path.join('autoapi', 'management', 'commands', 'data', self.type, f"import{self.file_extension}")

    def initialize_names(self):
        self.viewset_name = f"{self.model.lower()}Viewsets"
        self.model_serializer = f"{self.model.lower()}_serializers"
        self.list_serializer = f"{self.model}ListSerializers"
        self.retrieve_serializer = f"{self.model}RetrieveSerializers"
        self.write_serializer = f"{self.model}WriteSerializers"
        self.model_permission_class = f"{self.model.title()}{self.type.title()}"
        self.api_endpoint = self.model.lower()
        self.router_viewset_name = f"{self.model.lower()}_viewsets"

    def is_exists(self):
        return os.path.exists(self.model_type_stru)

    def code_exists(self):
        if os.path.exists(self.model_type_stru):
            with open(self.model_type_stru, 'r') as file:
                content = file.read()
                if self.viewset_name in content:
                    return True
        return False

    def formatter(self, base_data):
        return base_data.format(
            router_viewset_name=self.router_viewset_name,
            api_endpoint=self.api_endpoint,
            app_name=self.app_name,
            viewset_name=self.viewset_name,
            model_name=self.model,
            model_serializer=self.model_serializer,
            model_list_serializers=self.list_serializer,
            model_retrieve_serializers=self.retrieve_serializer,
            model_write_serializers=self.write_serializer,
            model_permission=self.model_permission_class,
        )

    def import_class(self):
        with open(self.model_type_date_file_backup_import, 'r') as file:
            data = file.read()
        formatted_data = self.formatter(data)
        with open(self.model_type_stru, 'r') as file:
            lines = file.readlines()
        with open(self.model_type_stru, 'w') as file:
            lines.insert(2, f"{formatted_data}\n")
            file.writelines(lines)

    def create(self):
        os.makedirs(os.path.dirname(self.model_type_stru), exist_ok=True)
        if self.type not in self.backup_types or not self.is_exists():
            with open(self.model_type_data_file, 'r') as file:
                base_data = file.read()
        else:
            if self.code_exists():
                return False
            with open(self.model_type_date_file_backup, 'r') as file:
                base_data = file.read()
            self.import_class()

        formatted_data = self.formatter(base_data)
        with open(self.model_type_stru, 'a') as file:
            file.write(formatted_data)
        
        return True

    def run(self):
        try:
            viewset_obj = ViewSetsHelper(self.app_name, self.model)
            viewsets = viewset_obj.find_viewsets_for_model()
        except:
            viewsets = None

        if viewsets:
            self.stdout.write(self.style.ERROR(f'{self.type} for model {self.model} viewset name {viewsets[1][2]} already exists!'))
        else:
            if self.is_exists() and self.type not in self.backup_types:
                self.stdout.write(self.style.ERROR(f'{self.type} for {self.model} models already exists!'))
            else:
                if self.create():
                    print("create")
                    self.stdout.write(self.style.SUCCESS(f'{self.type} for {self.model} models created successfully!'))

        




from django.core.management.base import BaseCommand, CommandError
# from polls.models import Question as Poll


class Command(BaseCommand):
    help = "Closes the specified poll for voting"

    def add_arguments(self, parser):
        parser.add_argument("--app", nargs="+", type=str)

    def handle(self, *args, **options):
        if options.get('app'):
            for app in options["app"]:
                # raise CommandError('Poll "%s" does not exist' % poll_id)
                self.stdout.write(
                    self.style.SUCCESS('Successfully maked api for app "%s"' % app)
                )
        else:
            self.stdout.write(
                    self.style.SUCCESS("Successfully restored api of apps")
                )
            


from rest_framework.routers import DefaultRouter
from ..viewsets.{router_viewset_name} import {viewset_name}

router = DefaultRouter()
auto_api_routers = router


router.register('{api_endpoint}', {viewset_name}, basename="{viewset_name}")



#------permission are classified into three types------->
#first-level:-Admin,Superadmin,Superuser (this is  user model class which can be considered as ObjectA)
#second-level:-object 'B' is assigned to user(i.e in object B , ObjectA is assigned), where user called as ObjectA
#third-level:-object 'C' is assigned to object object B(i.e object B is assigned in object C)

#model ObjectB->user field
#model ObjectC->objectB field(objectB id)


#as example, we can consider as , user,company,job where user is ObjectA,company is ObjectB,job is ObjectC

from rest_framework.permissions import BasePermission
from accounts import roles

def IsAuthenticated(request):
    return bool(request.user and request.user.is_authenticated)

def SuperAdminLevel(request):
    return bool(IsAuthenticated(request) and request.user.is_superuser)

def AdminLevel(request):
    return bool(IsAuthenticated(request) and request.user.role in [roles.ADMIN,roles.SUPER_ADMIN])

def isOwner(request):
    if str(request.user.id) == str(request.data.get('user')):
        return True
    
    elif len(request.data)==0 and len(request.POST)==0:
        return True

    return False


# def ObjectBOwner(request):
#     company = ObjectB.objects.filter(id = request.data.get('objectb'),user = request.user.id)
#     if company.exists():
#         return True
#     return False

class {app_name}Permission(BasePermission):
    def has_permission(self, request, view):
        if view.action in ["list"]:
            return True
        elif view.action in ['retrieve']:
            return isOwner(request)
        elif view.action in ['create','update']:
            return isOwner(request) #second level
            return ObjectBOwner(request) #third level
        elif view.action == "partial_update":
            return view.get_object().user_id == request.user.id
        elif view.action == 'destroy':
            return isOwner(request)




from setuptools import setup, find_packages
import os

# Read the content of your README file
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='drfapigenerator',
    version='2.0.0',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=3.2',
        'djangorestframework',
        'django-filter',
    ],
    entry_points={
        'console_scripts': [
            # Define any command line scripts here
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Framework :: Django',
        'Framework :: Django :: 3.2',  # Specify the appropriate Django version
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    long_description=long_description,
    long_description_content_type='text/markdown',
)



from rest_framework import serializers
from ..models import {model_name}

class {model_list_serializers}(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = '__all__'

class {model_retrieve_serializers}(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = '__all__'

class {model_write_serializers}(serializers.ModelSerializer):
    class Meta:
        model = {model_name}
        fields = '__all__'